/*
 * atmel.h -- USB Atmel programmer Atmel device. 
 * 
 * Copyright (c) 2005--2009 by Wolfgang Wieser ] wwieser (a) gmx <*> de [
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation. (See COPYING.GPL for details.)
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef _USB_ATMEL_PRG_ATMEL_H_
#define _USB_ATMEL_PRG_ATMEL_H_ 1

#include "../cable/usbio.h"
#include <stdio.h>

// See buffer.h: 
struct DataBuffer;


struct AtmelDevice
{
	const char *vendor;  // "Atmel"
	const char *name;    // "ATMega8"
	
	int flash_size;      // in WORDS
	int eeprom_size;     // in BYTES
	
	int page_size;       // in BYTES or 0
	int fuse_lock_bytes; // number of bytes
	bool has_erase_busy;
	int calibration_bytes;  // number of calibration bytes
	
	void *operator new(size_t size);
	void operator delete(void *ptr);
	
	AtmelDevice(const char *_vendor,const char *_name);
	virtual ~AtmelDevice() {}
	
	virtual void DumpIdentification(FILE *out);
	
	virtual int EraseAll();
	
	virtual int ReadFlash(DataBuffer *db,int offset,int nwords);
	int ReadFlash(DataBuffer *db)  // All flash. 
		{  return(ReadFlash(db,0,flash_size));  }
	virtual int WriteFlash(const DataBuffer *db,int offset);
	
	virtual int ReadEEPROM(DataBuffer *db,int offset,int nbytes);
	int ReadEEPROM(DataBuffer *db)  // All EEPROM. 
		{  return(ReadEEPROM(db,0,eeprom_size));  }
	virtual int WriteEEPROM(const DataBuffer *db,int offset);
	virtual int EraseEEPROM();
	
	virtual int DumpFuseLock(FILE *out);
	virtual int WriteFuseLock(FILE *out,char *const *args,
		int nargs);
	
	// Internally used: 
	virtual void _DumpLock(FILE *out,uint8 lock);
	virtual void _DumpFuseL(FILE *out,uint8 fuse);
	virtual void _DumpFuseH(FILE *out,uint8 fuseH);
	virtual void _DumpFuseX(FILE *out,uint8 fuseH);
	virtual void _DumpCalib(FILE *out,int ncalib,uint8 calib[]);
	// Internally used, non-virtual: 
	int _DoDumpFuseLock(FILE *out,int fuse_select_mask);
	int _DoWriteFuseLock(FILE *out,int fuse_select_mask,
		char *const *args,int nargs);
	// Internally used, non-virtual: 
	int _DoDumpFuseLock_L_F_FH_FX(FILE *out);
	
	// Dump a list of supported devices. 
	static void DumpSupportedDevices(FILE *out);
};

// Get the device struct by signature. Returns NULL if unknown / not supported. 
AtmelDevice *GetDeviceBySignature(USBAtmelSignature *sig);

#endif  /*_USB_ATMEL_PRG_ATMEL_H_ */
